package tls

import (
	"errors"
	"net"

	utls "github.com/refraction-networking/utls"
	"gitlab.torproject.org/tpo/anti-censorship/pluggable-transports/snowflake/v2/common/certs"
)

type Config struct {
	ServerName string
}

func NewTLSTransport(config *Config) (Transport, error) {
	return Transport{kind: "tls", serverName: config.ServerName}, nil
}

type Transport struct {
	kind       string
	serverName string
}

func (t Transport) Client(conn net.Conn) (net.Conn, error) {
	switch t.kind {
	case "tls":
		utlsClientHelloID := utls.HelloRandomizedNoALPN
		utlsConfig := &utls.Config{
			RootCAs:    certs.GetRootCAs(),
			ServerName: t.serverName,
		}
		return utls.UClient(conn, utlsConfig, utlsClientHelloID), nil
	}
	return nil, errors.New("unknown kind")
}
